@@ -1,17 +1,23 @@
 import unittest
 
-from urllib3.connectionpool import connection_from_url, HTTPConnectionPool
+from urllib3.connectionpool import (
+    connection_from_url,
+    HTTPConnection,
+    HTTPConnectionPool,
+)
+from urllib3.util.timeout import Timeout
 from urllib3.packages.ssl_match_hostname import CertificateError
 from urllib3.exceptions import (
     ClosedPoolError,
     EmptyPoolError,
     HostChangedError,
+    LocationValueError,
     MaxRetryError,
+    ProtocolError,
     SSLError,
-    TimeoutError,
 )
 
-from socket import error as SocketError, timeout as SocketTimeout
+from socket import error as SocketError
 from ssl import SSLError as BaseSSLError
 
 try:   # Python 3
@@ -34,6 +40,11 @@ class TestConnectionPool(unittest.TestCase):
             ('http://google.com/', 'http://google.com'),
             ('http://google.com/', 'http://google.com/abra/cadabra'),
             ('http://google.com:42/', 'http://google.com:42/abracadabra'),
+            # Test comparison using default ports
+            ('http://google.com:80/', 'http://google.com/abracadabra'),
+            ('http://google.com/', 'http://google.com:80/abracadabra'),
+            ('https://google.com:443/', 'https://google.com/abracadabra'),
+            ('https://google.com/', 'https://google.com:443/abracadabra'),
         ]
 
         for a, b in same_host:
@@ -46,11 +57,23 @@ class TestConnectionPool(unittest.TestCase):
             ('http://yahoo.com/', 'http://google.com/'),
             ('http://google.com:42', 'https://google.com/abracadabra'),
             ('http://google.com', 'https://google.net/'),
+            # Test comparison with default ports
+            ('http://google.com:42', 'http://google.com'),
+            ('https://google.com:42', 'https://google.com'),
+            ('http://google.com:443', 'http://google.com'),
+            ('https://google.com:80', 'https://google.com'),
+            ('http://google.com:443', 'https://google.com'),
+            ('https://google.com:80', 'http://google.com'),
+            ('https://google.com:443', 'http://google.com'),
+            ('http://google.com:80', 'https://google.com'),
         ]
 
         for a, b in not_same_host:
             c = connection_from_url(a)
             self.assertFalse(c.is_same_host(b), "%s =? %s" % (a, b))
+            c = connection_from_url(b)
+            self.assertFalse(c.is_same_host(a), "%s =? %s" % (b, a))
+
 
     def test_max_connections(self):
         pool = HTTPConnectionPool(host='localhost', maxsize=1, block=True)
@@ -95,7 +118,7 @@ class TestConnectionPool(unittest.TestCase):
             str(MaxRetryError(
                 HTTPConnectionPool(host='localhost'), "Test.", None)),
             "HTTPConnectionPool(host='localhost', port=None): "
-            "Max retries exceeded with url: Test. (Caused by redirect)")
+            "Max retries exceeded with url: Test. (Caused by None)")
 
         err = SocketError("Test")
 
@@ -106,7 +129,8 @@ class TestConnectionPool(unittest.TestCase):
                 HTTPConnectionPool(host='localhost'), "Test.", err)),
             "HTTPConnectionPool(host='localhost', port=None): "
             "Max retries exceeded with url: Test. "
-            "(Caused by {0}: Test)".format(str(err.__class__)))
+            "(Caused by %r)" % err)
+
 
     def test_pool_size(self):
         POOL_SIZE = 1
@@ -121,9 +145,8 @@ class TestConnectionPool(unittest.TestCase):
 
             self.assertEqual(pool.pool.qsize(), POOL_SIZE)
 
-        #make sure that all of the exceptions return the connection to the pool
-        _test(Empty, TimeoutError)
-        _test(SocketTimeout, TimeoutError)
+        # Make sure that all of the exceptions return the connection to the pool
+        _test(Empty, EmptyPoolError)
         _test(BaseSSLError, SSLError)
         _test(CertificateError, SSLError)
 
@@ -165,6 +188,23 @@ class TestConnectionPool(unittest.TestCase):
 
         self.assertRaises(Empty, old_pool_queue.get, block=False)
 
+    def test_pool_timeouts(self):
+        pool = HTTPConnectionPool(host='localhost')
+        conn = pool._new_conn()
+        self.assertEqual(conn.__class__, HTTPConnection)
+        self.assertEqual(pool.timeout.__class__, Timeout)
+        self.assertEqual(pool.timeout._read, Timeout.DEFAULT_TIMEOUT)
+        self.assertEqual(pool.timeout._connect, Timeout.DEFAULT_TIMEOUT)
+        self.assertEqual(pool.timeout.total, None)
+
+        pool = HTTPConnectionPool(host='localhost', timeout=3)
+        self.assertEqual(pool.timeout._read, 3)
+        self.assertEqual(pool.timeout._connect, 3)
+        self.assertEqual(pool.timeout.total, None)
+
+    def test_no_host(self):
+        self.assertRaises(LocationValueError, HTTPConnectionPool, None)
+
 
 
 if __name__ == '__main__':
